
## Diebold-Mariano test (https://doi.org/10.2307/1392185)
library(SpecsVerification)
library(ClimProjDiags)
library(multiApply)

DieboldMarianoTest <- function(skill_A, skill_B, obs = NA, time_dim = 'year', alpha = 0.05, N.eff = NA, ncores = 1){
  
  ## Check inputs
  if (is.null(skill_A) | is.null(skill_B)){
    stop("Parameters 'skill_A' and 'skill_B' cannot be NULL.")
  }
  if(!is.numeric(skill_A) | !is.numeric(skill_B)){
    stop("Parameters 'skill_A' and 'skill_B' must be a numerical array.")
  }
  if (!identical(dim(skill_A),dim(skill_B))){
    stop("Parameters 'skill_A' and 'skill_B' must have the same dimensions.")
  }
  if(!is.character(time_dim)){
    stop("Parameter 'time_dim' must be a character string.")
  }
  if(!time_dim %in% names(dim(skill_A)) | !time_dim %in% names(dim(skill_B))){
    stop("Parameter 'time_dim' is not found in 'skill_A' or 'skill_B' dimensions.")
  }
  if(!is.numeric(alpha) | length(alpha) != 1 | alpha >= 1 | alpha <= 0){
    stop("Parameter 'alpha' must be numeric, of length 1, higher than 0, and lower than 1.")
  }
  if(!identical(N.eff,NA)){
    if (!identical(dim(N.eff), dim(skill_A)[-which(names(dim(skill_A)) == time_dim)]) | !is.numeric(N.eff)){
      stop("Parameter 'N.eff' must be NA, or a numerical array with the same dimensions as 'skill_A' and 'skill_B' but without the 'time_dim' dimension")
    }
  }
  if (!is.null(ncores)){
    if (!is.numeric(ncores) | ncores %% 1 != 0 | ncores <= 0 | length(ncores) > 1){
      stop("Parameter 'ncores' must be a positive integer.")
    }
  }
  
  ## Compute the Diebold-Mariano Test
  res <- multiApply::Apply(data = list(skill_A = skill_A, skill_B = skill_B, N.eff = N.eff), 
                           target_dims = list(skill_A = time_dim, skill_B = time_dim, N.eff = NULL), 
                           fun = .DieboldMarianoTest, obs = obs, alpha = alpha, ncores = ncores)$output1
  return(res)
}

.DieboldMarianoTest <- function(skill_A, skill_B, obs = NA, N.eff = NA, alpha = 0.05){
  if (anyNA(skill_A) | anyNA(skill_B)){
    return(NA)
  } else {
    
    if (is.na(N.eff)){
      N.eff <- s2dv:::.Eno(x = obs, na.action = na.omit)
    } else if (isFALSE(N.eff)){
      N.eff <- length(skill_A)
    }
    
    print(N.eff)
    
    ## one-sided
    # score_diff <- SpecsVerification::ScoreDiff(scores = skill_A, scores.ref = skill_B, N.eff = N.eff, conf.level = 1-alpha, handle.na = 'na.fail')
    # return(score_diff[3] < alpha)
    
    ## two-sided
    score_diff <- SpecsVerification_ScoreDiff_twosided(scores = skill_A, scores.ref = skill_B, N.eff = N.eff, conf.level = 1-alpha, handle.na = 'na.fail')
    return(list(sign = score_diff[3] < alpha/2))
  }
}

SpecsVerification_ScoreDiff_twosided <- function (scores, scores.ref, N.eff = NA, conf.level = 0.95, handle.na = "na.fail") 
{
  N.eff <- N.eff[1L]
  stopifnot(N.eff > 0 | is.na(N.eff))
  stopifnot(length(scores) == length(scores.ref))
  if (handle.na == "na.fail") {
    if (any(is.na(c(scores, scores.ref)))) {
      stop("missing values")
    }
  }
  else if (handle.na == "use.pairwise.complete") {
    nna <- !is.na(scores) & !is.na(scores.ref)
    if (all(nna == FALSE)) {
      stop("there are no complete pairs of scores")
    }
    scores <- scores[nna]
    scores.ref <- scores.ref[nna]
  }
  else {
    stop("unknown 'handle.na' argument")
  }
  if (is.na(N.eff)) {
    N.eff <- length(scores)
  }
  d <- scores.ref - scores
  d.bar <- abs(mean(d)) ## abs() to get the significance also for the negative RPSS
  var.d <- var(d)
  d.bar.sd <- sqrt(var.d/N.eff)
  if (d.bar.sd > 0) {
    p.value <- pnorm(d.bar/d.bar.sd, lower.tail = FALSE)
  }
  else {
    p.value <- NA
  }
  if (conf.level <= 0 | conf.level >= 1) {
    conf.level <- NA
  }
  if (is.na(conf.level)) {
    ci <- c(NA, NA)
  }
  else {
    alpha <- (1 - conf.level)/2
    ci <- qnorm(p = c(alpha, 1 - alpha), mean = d.bar, sd = d.bar.sd)
  }
  ret <- c(score.diff = d.bar, score.diff.sd = d.bar.sd, p.value = p.value, 
           L = ci[1], U = ci[2])
  return(ret)
}
